""" Module for canonical-mode tests. """
# std imports
import sys
import os

# local
import pexpect
from . import PexpectTestCase

# 3rd-party
import pytest


class TestCaseCanon(PexpectTestCase.PexpectTestCase):
    """
    Test expected Canonical mode behavior (limited input line length).

    All systems use the value of MAX_CANON which can be found using
    fpathconf(3) value PC_MAX_CANON -- with the exception of Linux
    and FreeBSD.

    Linux, though defining a value of 255, actually honors the value
    of 4096 from linux kernel include file tty.h definition
    N_TTY_BUF_SIZE.

    Linux also does not honor IMAXBEL. termios(3) states, "Linux does not
    implement this bit, and acts as if it is always set." Although these
    tests ensure it is enabled, this is a non-op for Linux.

    FreeBSD supports neither, and instead uses a fraction (1/5) of the tty
    speed which is always 9600.  Therefor, the maximum limited input line
    length is 9600 / 5 = 1920.

    These tests only ensure the correctness of the behavior described by
    the sendline() docstring. pexpect is not particularly involved in
    these scenarios, though if we wish to expose some kind of interface
    to tty.setraw, for example, these tests may be re-purposed as such.

    Lastly, portions of these tests are skipped on Travis-CI. It produces
    unexpected behavior not reproduced on Debian/GNU Linux.
    """

    def setUp(self):
        super(TestCaseCanon, self).setUp()

        self.echo = False
        if sys.platform.lower().startswith('linux'):
            # linux is 4096, N_TTY_BUF_SIZE.
            self.max_input = 4096
            self.echo = True
        elif sys.platform.lower().startswith('sunos'):
            # SunOS allows PC_MAX_CANON + 1; see
            # https://bitbucket.org/illumos/illumos-gate/src/d07a59219ab7fd2a7f39eb47c46cf083c88e932f/usr/src/uts/common/io/ldterm.c?at=default#cl-1888
            self.max_input = os.fpathconf(0, 'PC_MAX_CANON') + 1
        elif sys.platform.lower().startswith('freebsd'):
            # http://lists.freebsd.org/pipermail/freebsd-stable/2009-October/052318.html
            self.max_input = 9600 / 5
        else:
            # All others (probably) limit exactly at PC_MAX_CANON
            self.max_input = os.fpathconf(0, 'PC_MAX_CANON')

    @pytest.mark.skipif(
        sys.platform.lower().startswith('freebsd'),
        reason='os.write to BLOCK indefinitely on FreeBSD in this case'
    )
    def test_under_max_canon(self):
        " BEL is not sent by terminal driver at maximum bytes - 1. "
        # given,
        child = pexpect.spawn('bash', echo=self.echo, timeout=5)
        child.sendline('echo READY')
        child.sendline('stty icanon imaxbel')
        child.sendline('echo BEGIN; cat')

        # some systems BEL on (maximum - 1), not able to receive CR,
        # even though all characters up until then were received, they
        # simply cannot be transmitted, as CR is part of the transmission.
        send_bytes = self.max_input - 1

        # exercise,
        child.sendline('_' * send_bytes)

        # fast forward beyond 'cat' command, as ^G can be found as part of
        # set-xterm-title sequence of $PROMPT_COMMAND or $PS1.
        child.expect_exact('BEGIN')

        # verify, all input is found in echo output,
        child.expect_exact('_' * send_bytes)

        # BEL is not found,
        with self.assertRaises(pexpect.TIMEOUT):
            child.expect_exact('\a', timeout=1)

        # cleanup,
        child.sendeof()           # exit cat(1)
        child.sendline('exit 0')  # exit bash(1)
        child.expect(pexpect.EOF)
        assert not child.isalive()
        assert child.exitstatus == 0

    @pytest.mark.skipif(
        sys.platform.lower().startswith('freebsd'),
        reason='os.write to BLOCK indefinitely on FreeBSD in this case'
    )
    def test_beyond_max_icanon(self):
        " a single BEL is sent when maximum bytes is reached. "
        # given,
        child = pexpect.spawn('bash', echo=self.echo, timeout=5)
        child.sendline('stty icanon imaxbel erase ^H')
        child.sendline('cat')
        send_bytes = self.max_input

        # exercise,
        child.sendline('_' * send_bytes)
        child.expect_exact('\a')

        # exercise, we must now backspace to send CR.
        child.sendcontrol('h')
        child.sendline()

        if os.environ.get('TRAVIS', None) == 'true':
            # Travis-CI has intermittent behavior here, possibly
            # because the master process is itself, a PTY?
            return

        # verify the length of (maximum - 1) received by cat(1),
        # which has written it back out,
        child.expect_exact('_' * (send_bytes - 1))
        # and not a byte more.
        with self.assertRaises(pexpect.TIMEOUT):
            child.expect_exact('_', timeout=1)

        # cleanup,
        child.sendeof()           # exit cat(1)
        child.sendline('exit 0')  # exit bash(1)
        child.expect_exact(pexpect.EOF)
        assert not child.isalive()
        assert child.exitstatus == 0

    @pytest.mark.skipif(
        sys.platform.lower().startswith('freebsd'),
        reason='os.write to BLOCK indefinitely on FreeBSD in this case'
    )
    def test_max_no_icanon(self):
        " may exceed maximum input bytes if canonical mode is disabled. "
        # given,
        child = pexpect.spawn('bash', echo=self.echo, timeout=5)
        child.sendline('stty -icanon imaxbel')
        child.sendline('echo BEGIN; cat')
        send_bytes = self.max_input + 11

        # exercise,
        child.sendline('_' * send_bytes)

        # fast forward beyond 'cat' command, as ^G can be found as part of
        # set-xterm-title sequence of $PROMPT_COMMAND or $PS1.
        child.expect_exact('BEGIN')

        if os.environ.get('TRAVIS', None) == 'true':
            # Travis-CI has intermittent behavior here, possibly
            # because the master process is itself, a PTY?
            return

        # BEL is *not* found,
        with self.assertRaises(pexpect.TIMEOUT):
            child.expect_exact('\a', timeout=1)

        # verify, all input is found in output,
        child.expect_exact('_' * send_bytes)

        # cleanup,
        child.sendcontrol('c')    # exit cat(1) (eof wont work in -icanon)
        child.sendcontrol('c')
        child.sendline('exit 0')  # exit bash(1)
        child.expect(pexpect.EOF)
        assert not child.isalive()
        assert child.exitstatus == 0



#!/usr/bin/env python
'''
PEXPECT LICENSE

    This license is approved by the OSI and FSF as GPL-compatible.
        http://opensource.org/licenses/isc-license.txt

    Copyright (c) 2012, Noah Spurrier <noah@noah.org>
    PERMISSION TO USE, COPY, MODIFY, AND/OR DISTRIBUTE THIS SOFTWARE FOR ANY
    PURPOSE WITH OR WITHOUT FEE IS HEREBY GRANTED, PROVIDED THAT THE ABOVE
    COPYRIGHT NOTICE AND THIS PERMISSION NOTICE APPEAR IN ALL COPIES.
    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

'''
import pexpect
import unittest
import os
import tempfile
from . import PexpectTestCase

# the program cat(1) may display ^D\x08\x08 when \x04 (EOF, Ctrl-D) is sent
_CAT_EOF = b'^D\x08\x08'

class TestCaseLog(PexpectTestCase.PexpectTestCase):

    def test_log (self):
        log_message = 'This is a test.'
        filename = tempfile.mktemp()
        mylog = open(filename, 'wb')
        p = pexpect.spawn('echo', [log_message])
        p.logfile = mylog
        p.expect(pexpect.EOF)
        p.logfile = None
        mylog.close()
        with open(filename, 'rb') as f:
            lf = f.read()
        os.unlink(filename)
        self.assertEqual(lf.rstrip(), log_message.encode('ascii'))

    def test_log_logfile_read (self):
        log_message = 'This is a test.'
        filename = tempfile.mktemp()
        mylog = open(filename, 'wb')
        p = pexpect.spawn('cat')
        p.logfile_read = mylog
        p.sendline(log_message)
        p.sendeof()
        p.expect(pexpect.EOF)
        p.logfile = None
        mylog.close()
        with open(filename, 'rb') as f:
            lf = f.read()
        os.unlink (filename)
        lf = lf.replace(_CAT_EOF, b'')
        self.assertEqual(lf, b'This is a test.\r\nThis is a test.\r\n')

    def test_log_logfile_send (self):
        log_message = b'This is a test.'
        filename = tempfile.mktemp()
        mylog = open (filename, 'wb')
        p = pexpect.spawn('cat')
        p.logfile_send = mylog
        p.sendline(log_message)
        p.sendeof()
        p.expect (pexpect.EOF)
        p.logfile = None
        mylog.close()
        with open(filename, 'rb') as f:
            lf = f.read()
        os.unlink(filename)
        lf = lf.replace(b'\x04', b'')
        self.assertEqual(lf.rstrip(), log_message)

    def test_log_send_and_received (self):

        '''The logfile should have the test message three time -- once for the
        data we sent. Once for the data that cat echos back as characters are
        typed. And once for the data that cat prints after we send a linefeed
        (sent by sendline). '''

        log_message = 'This is a test.'
        filename = tempfile.mktemp()
        mylog = open(filename, 'wb')
        p = pexpect.spawn('cat')
        p.logfile = mylog
        p.sendline(log_message)
        p.sendeof()
        p.expect (pexpect.EOF)
        p.logfile = None
        mylog.close()
        with open(filename, 'rb') as f:
            lf = f.read()
        os.unlink(filename)
        lf = lf.replace(b'\x04', b'').replace(_CAT_EOF, b'')
        self.assertEqual(lf,
                b'This is a test.\nThis is a test.\r\nThis is a test.\r\n')

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(TestCaseLog,'test')




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
PEXPECT LICENSE

    This license is approved by the OSI and FSF as GPL-compatible.
        http://opensource.org/licenses/isc-license.txt

    Copyright (c) 2012, Noah Spurrier <noah@noah.org>
    PERMISSION TO USE, COPY, MODIFY, AND/OR DISTRIBUTE THIS SOFTWARE FOR ANY
    PURPOSE WITH OR WITHOUT FEE IS HEREBY GRANTED, PROVIDED THAT THE ABOVE
    COPYRIGHT NOTICE AND THIS PERMISSION NOTICE APPEAR IN ALL COPIES.
    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
    WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
    MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
    ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
    WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
    ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
    OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

'''
from __future__ import print_function
from __future__ import unicode_literals

import os
import pexpect
import unittest
import sys
from . import PexpectTestCase


class InteractTestCase (PexpectTestCase.PexpectTestCase):
    def setUp(self):
        super(InteractTestCase, self).setUp()
        self.env = env = os.environ.copy()

        # Ensure 'import pexpect' works in subprocess interact*.py
        if 'PYTHONPATH' in env:
            env['PYTHONPATH'] = os.pathsep.join((self.project_dir,
                                                    env['PYTHONPATH']))
        else:
            env['PYTHONPATH'] = self.project_dir

        self.interact_py = ('{sys.executable} interact.py'.format(sys=sys))

    def test_interact_escape(self):
        " Ensure `escape_character' value exits interactive mode. "
        p = pexpect.spawn(self.interact_py, timeout=5, env=self.env)
        p.expect('READY')
        p.sendcontrol(']')  # chr(29), the default `escape_character'
                            # value of pexpect.interact().
        p.expect_exact('Escaped interact')
        p.expect(pexpect.EOF)
        assert not p.isalive()
        assert p.exitstatus == 0

    def test_interact_escape_None(self):
        " Return only after Termination when `escape_character=None'. "
        p = pexpect.spawn('{self.interact_py} --no-escape'.format(self=self),
                          timeout=5, env=self.env)
        p.expect('READY')
        p.sendcontrol(']')
        p.expect('29<STOP>')
        p.send('\x00')
        if not os.environ.get('TRAVIS', None):
            # on Travis-CI, we sometimes miss trailing stdout from the
            # chain of child processes, not entirely sure why. So this
            # is skipped on such systems.
            p.expect('0<STOP>')
            p.expect_exact('Escaped interact')
        p.expect(pexpect.EOF)
        assert not p.isalive()
        assert p.exitstatus == 0

    def test_interact_exit_unicode(self):
        " Ensure subprocess receives utf8. "
        p = pexpect.spawnu('{self.interact_py} --utf8'.format(self=self),
                           timeout=5, env=self.env)
        p.expect('READY')
        p.send('ɑ')              # >>> map(ord, u'ɑ'.encode('utf8'))
        p.expect('201<STOP>')    # [201, 145]
        p.expect('145<STOP>')
        p.send('Β')              # >>> map(ord, u'Β'.encode('utf8'))
        p.expect('206<STOP>')    # [206, 146]
        p.expect('146<STOP>')
        p.send('\x00')
        if not os.environ.get('TRAVIS', None):
            # on Travis-CI, we sometimes miss trailing stdout from the
            # chain of child processes, not entirely sure why. So this
            # is skipped on such systems.
            p.expect('0<STOP>')
            p.expect_exact('Escaped interact')
        p.expect(pexpect.EOF)
        assert not p.isalive()
        assert p.exitstatus == 0

if __name__ == '__main__':
    unittest.main()

suite = unittest.makeSuite(InteractTestCase, 'test')

